/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IUpdateNeuedaten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse implementiert eine reale Anzeige vom Typ 'typ.anzeige'.<p>
 * Empfangen und verwalten der aktuellen Rckmeldungen und Fehler <code>atg.anzeigeEingenschaftIst</code>. 
 *
 * @version $Revision: 1.3 $ / $Date: 2015/09/02 12:49:38 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class Anzeige implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static final Debug m_debug = Debug.getLogger();
	/**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavConnection m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Anzeigequerschnitt.
	 */
	private SystemObject m_anzeige;
	/**
	 * AnzeigeTyp der Anzeige.
	 */
	private SystemObject m_anzeigeTyp;	
	/**
	 * Liste der an dieser Anzeige vorhandenen WVZ-Inhalte.
	 */
	private Vector<String> m_wvzInhalte;
	
	/**
	 * In dieser Map sind die Pid's der WVZ-Inhalte abgelegt.
	 * Key ist der zugehrige Bildinhalt.
	 */
	private HashMap<String, String> m_pidWvzInhalt = new HashMap<String, String>();
	/**
	 * Attributgruppenverwendungen Anzeige.
	 */
	private final String[][] m_UeEmpfaenger = new String[][]
  {
	  // Attributgruppe, Aspekt
	  {"atg.anzeigeEigenschaftIst", "asp.aqZustand"}    
	};

	/**
	 * Aktueller Status der Anzeige.
	 */
	private AnzeigenStatus m_anzeigenStatus;
	
	/**
	 * Liste der dieser Anzeige zugeordneten Fahrstreifen.
	 */
	private List<SystemObject> m_fahrstreifen;
	/**
	 * Listener neue Daten erhalten.
	 */
	private Vector<IUpdateNeuedaten> m_listener = new Vector<IUpdateNeuedaten>();
	/**
	 * Lage der Anzeige bezogen auf den Fahrstreifen.
	 */
	private String m_lage;
	
	/**
	 * Konstruktor mit bernahme der Parameter.
	 * 
	 * @param anzeige Systemobjekt Anzeige.
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 */
	public Anzeige(SystemObject anzeige, ClientDavConnection connection)
	{
		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		m_anzeige = anzeige;
		
		m_wvzInhalte = new Vector<String>();
		
		bestimmeAnzeigeTyp(anzeige);
		bestimmeWvzInhalte(m_anzeigeTyp);
		
//		datenDavAnmelden();
	}
	
	/**
	 * Diese Methode bestimmt den Anzeigetyp der Anzeige.
	 * 
	 * @param anzeige Objekt Anzeige.
	 */
	private void bestimmeAnzeigeTyp(SystemObject anzeige)
	{
		if (anzeige != null)
		{
			ConfigurationObject co = (ConfigurationObject)anzeige;
			if (co != null)
			{
				Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.anzeige"));
				if (data != null)
				{
					m_anzeigeTyp = data.getReferenceValue("AnzeigeTyp").getSystemObject();
				}
			}
		}
		
	}

	/**
	 * Diese Methode bestimmt ber den bergebenen Anzeigetyp, die WVZ-Inhalte, die an dieser Anzeige 
	 * verfgbar sind und schreibt diese in einen globale Vektor.
	 * 
	 * @param anzeigeTyp Objekt Anzeigetyp.
	 */
	private void bestimmeWvzInhalte(SystemObject anzeigeTyp)
	{
		if (anzeigeTyp != null)
		{
			ConfigurationObject co = (ConfigurationObject)anzeigeTyp;
			if (co != null)
			{
				List<SystemObject> wvzInhalte = co.getNonMutableSet("WvzInhalt").getElements();
				if (wvzInhalte != null)
				{
					for (SystemObject wvzInhalt : wvzInhalte)
					{
						ConfigurationObject coWvzInhalt = (ConfigurationObject)wvzInhalt;
						if (coWvzInhalt != null)
						{
							Data data = coWvzInhalt.getConfigurationData(m_dataModel.getAttributeGroup("atg.wvzInhalt"));
							if (data != null)
							{
								String bildinhalt = data.getTextValue("Bildinhalt").getValueText();
								
								m_wvzInhalte.add(bildinhalt);
								m_pidWvzInhalt.put(bildinhalt, new String(wvzInhalt.getPid()));
							}
						}
					}
				}
			}
		}
		
	}

	/**
	 * Alle bentigten Attributgruppenverwendungen am Datenverteiler anmelden.
	 */
	public void datenDavAnmelden()
	{
		// Empfnger Anlage anmelden
		for (int i = 0; i < m_UeEmpfaenger.length; i++)
		{
		  datenAnmeldenObjekt(this, m_anzeige, m_UeEmpfaenger[i][0], m_UeEmpfaenger[i][1]);
		}
	}
  /**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  public boolean datenAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, String atg, String asp)
  {
    AttributeGroup atgParameter;
    Aspect aspParameter;
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekte " + atg + " " + asp + " fr " + objekt));
    
    try
    {
      atgParameter = m_dataModel.getAttributeGroup(atg);
      aspParameter = m_dataModel.getAspect(asp);
    
      dd = new DataDescription(atgParameter, aspParameter);

      m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
      
      ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Senden angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] arg0)
	{
    for (int i = 0; i < arg0.length; i++) 
    {
    	ResultData rdata = arg0[i];

    	if (rdata.getDataDescription().getAttributeGroup().equals(m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst")))
    	{
 				m_anzeigenStatus = new AnzeigenStatus(rdata.getData());
 				
 				fireEvent();
 				
// 				System.out.println(m_anzeige.getPid() + " " + m_anzeigenStatus);
    	}
    }
	}
	
	/**
	 * Klasse zum Speichern des aktuellen Status der Anzeige.
	 *
	 * @version $Revision: 1.3 $ / $Date: 2015/09/02 12:49:38 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	private class AnzeigenStatus
	{
		/**
		 * Status der Anzeige<br>
		 * z.B. OK, Gestrt, Teilstrung usw.<br>
		 * bei Teilstrung zustzlich auf gestrte Inhalte prfen.
		 */
		private String m_status;
		
		/**
		 * Liste der gestrten WVZ-Inhalte.
		 */
		private Vector<String> m_gestoerteInhalte;

		/**
		 * Referenz auf den Anzeigeninhalt der Anzeige.
		 */
		private SystemObject m_anzeigenInhalt;		
		/**
		 * Datensatz Schaltbild.
		 */
		private Data m_schaltbildDaten;
		
		/**
		 * Konstruktor ohne weitere Funktion.
		 */
		public AnzeigenStatus()
		{			
		}
		
		/**
		 * Konstruktor mit bernahme der bergebenen Daten.
		 */
		public AnzeigenStatus(Data data)
		{
			if (data != null)
			{
				// Status
				m_status = data.getScaledValue("Status").getValueText();
				
				// gestrte Inhalte
				if (m_gestoerteInhalte == null)
					m_gestoerteInhalte = new Vector<String>();
				
				int anz = data.getReferenceArray("GestrteInhalte").getLength();
				for (int i = 0; i < anz; i++)
				{
					m_gestoerteInhalte.add(data.getReferenceArray("GestrteInhalte").getSystemObject(i).getPid());
				}
				
				m_anzeigenInhalt = data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").getSystemObject();
				m_schaltbildDaten = data.getItem("Eigenschaft");
			}
			else
			{
				m_status = "Undefiniert";

				if (m_gestoerteInhalte != null)
					m_gestoerteInhalte.clear();

				m_anzeigenInhalt = null;
				m_schaltbildDaten = null;
			}
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString()
		{
			StringBuffer string = new StringBuffer();
			
			string.append("Status : " + m_status + " gestrte Inhalte : " + m_gestoerteInhalte + " AnzeigeInhalt : " + m_anzeigenInhalt);
			
			return string.toString();
		}

		/**
		 * Gibt den aktuellen Status der Anzeige zurck.
		 * 
		 * @return Aktueller Status der Anzeige.
		 */
		public String getStatus()
		{
			return m_status;
		}

		/**
		 * Setzt den aktuellen Status der Anzeige.
		 * 
		 * @param status zu setzender Status.
		 */
		public void setStatus(String status)
		{
			m_status = status;
		}

		/**
		 * Gibt die Liste der gestrten Inhalte zurck.
		 * 
		 * @return Liste der gestrten Inhalte.
		 */
		public Vector<String> getGestoerteInhalte()
		{
			return m_gestoerteInhalte;
		}

		/**
		 * Setzt die Liste der gestrten Inhalte.
		 * 
		 * @param gestoerteInhalte zu setzende Liste der gestrten Inhalte.
		 */
		public void setGestoerteInhalte(Vector<String> gestoerteInhalte)
		{
			m_gestoerteInhalte = gestoerteInhalte;
		}

		/**
		 * Gibt das Systemobjekt des Anzeigeinhalt zurck.
		 * 
		 * @return Systemobjekt des Anzeigeinhalt.
		 */
		public SystemObject getAnzeigenInhalt()
		{
			return m_anzeigenInhalt;
		}

		/**
		 * Setzt das Systemobjekt des Anzeigeinhalt.
		 * @param anzeigenInhalt zu setzender Anzeigeinhalt.
		 */
		public void setAnzeigenInhalt(SystemObject anzeigenInhalt)
		{
			m_anzeigenInhalt = anzeigenInhalt;
		}

		/**
		 * Gibt den Datensatz der Schaltbilddaten zurck.
		 * 
		 * @return Datensatz der Schaltbilddaten.
		 */
		public Data getSchaltbildDaten()
		{
			return m_schaltbildDaten;
		}

		/**
		 * Setzt den Datensatz der Schaltbilddaten. 
		 * 
		 * @param schaltbildDaten zu setzender Datensatz.
		 */
		public void setSchaltbildDaten(Data schaltbildDaten)
		{
			m_schaltbildDaten = schaltbildDaten;
		}		
	}
	
	/**
	 * Gibt den Status der Anzeige zurck.
	 * 
	 * @return Status der Anzeige.
	 */
	public String getStatusAnzeige()
	{
		if (m_anzeigenStatus != null)
		{
			return m_anzeigenStatus.getStatus();
		}
		
		return null;
	}
	/**
	 * Gibt den aktuellen Anzeigeinhalt der Anzeige zurck.
	 * 
	 * @return Aktueller Anzeigeinhalt der Anzeige.
	 */
	public String getAnzeigeInhalt()
	{
		if (m_anzeigenStatus != null)
		{
			if (m_anzeigenStatus.getAnzeigenInhalt() != null)
				return m_anzeigenStatus.getAnzeigenInhalt().getPid();
			else
				return "undefiniert";
		}
		
		return "undefiniert";
	}
	/**
	 * Prft, ob die Anzeige eine Strung hat.
	 * 
	 * @return true wenn Anzeige eine Strung hat, ansonsten false.
	 */
	public boolean isAnzeigeGestoert()
	{
		if (m_anzeigenStatus != null)
		{
			String status = m_anzeigenStatus.getStatus();
			
			if ((status.equals("Gestrt")) ||
					(status.equals("Kommunikationsausfall")))
				return true;			
		}
		
		return false;
	}
	
	/**
	 * Prft, ob die Rckmeldung der Anzeige gesperrt (rotes Kreuz) ist.
	 * 
	 * @return true wenn Rckmeldung der Anzeige gesperrt (rotes Kreuz), ansonsten false.
	 */
	public boolean isAnzeigeGesperrt()
	{
		if (!isAnzeigeGestoert())
		{
			String pidAnzeigeInhaltGesperrt = m_pidWvzInhalt.get(Definitionen.KREUZ);
	
			if (pidAnzeigeInhaltGesperrt != null)
			{
				if (!isWvzInhaltGestoert(pidAnzeigeInhaltGesperrt))
				{
					if (getAnzeigeInhalt().equals(pidAnzeigeInhaltGesperrt))
					{
						return true;
					}				
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Gibt die Liste der gestrten Anzeigeninhalte zurck.
	 * 
	 * @return Liste der gestrten Anzeigeninhalte.
	 */
	public Vector<String> getGestoerteInhalte()
	{
		if (m_anzeigenStatus != null)
		{
			return m_anzeigenStatus.getGestoerteInhalte();
		}
		
		return null;
	}

	/**
	 * Gibt den Datensatz Schaltbild zurck.
	 * 
	 * @return Datensatz Schaltbild.
	 */
	public Data getSchaltbildDaten()
	{
		if (m_anzeigenStatus != null)
		{
			return m_anzeigenStatus.getSchaltbildDaten();
		}
		
		return null;
	}
	
	/**
	 * Prft, ob der bergebene WVZ-Inhalt in der Liste der nicht darstellbaren WVZ enthalten ist.
	 * 
	 * @param wvzInhalt PID des WVZ-Inhalt.
	 * @return true wenn WVZ-Inhalt in der Liste der nicht darstellbaren WVZ enthalten ist, ansonsten false.
	 */
	public boolean isWvzInhaltGestoert(String wvzInhalt)
	{
		if ((m_anzeigenStatus != null) && (wvzInhalt != null))
		{
			Vector<String> gestoerteInhalte = m_anzeigenStatus.getGestoerteInhalte();
			if (gestoerteInhalte != null)
			{
				for (int i = 0; i < gestoerteInhalte.size(); i++)
				{
					if (gestoerteInhalte.get(i).equals(wvzInhalt))
						return true;
				}
			}
		}
		
		return false;
	}

	/**
	 * Prft, ob an dieser Anzeige eine Spursperrung geschaltet werden kann.
	 * 
	 * @return true wenn eine Spursperrung geschaltet werden kann, ansonsten false.
	 */
	public boolean isSpurSperrungVorhanden()
	{
		if (m_wvzInhalte != null)
		{
			if (m_wvzInhalte.contains(Definitionen.KREUZ))
				return true;
		}
		return false;
	}
	/**
	 * Prft, ob an dieser Anzeige eine Aufhebung geschaltet werden kann.
	 * 
	 * @return true wenn eine Aufhebung geschaltet werden kann, ansonsten false.
	 */
	public boolean isAufhebungVorhanden()
	{
		if (m_wvzInhalte != null)
		{
			if ((m_wvzInhalte.contains(Definitionen.EALL)) ||
					(m_wvzInhalte.contains(Definitionen.EUALL)) ||
					(m_wvzInhalte.contains(Definitionen.EULKW)) ||
					(m_wvzInhalte.contains(Definitionen.EUPKW)))
				return true;
		}
		return false;
	}
	/**
	 * Prft, ob an dieser Anzeige eine Geschwindigkeitsbeschrnkung geschaltet werden kann.
	 * 
	 * @return true wenn eine Geschwindigkeitsbeschrnkung geschaltet werden kann, ansonsten false.
	 */
	public boolean isGeschwindigkeitVorhanden()
	{
		if (m_wvzInhalte != null)
		{
			if ((m_wvzInhalte.contains(Definitionen.V120)) ||
					(m_wvzInhalte.contains(Definitionen.V100)) ||
					(m_wvzInhalte.contains(Definitionen.V80)) ||
					(m_wvzInhalte.contains(Definitionen.V60)) ||
					(m_wvzInhalte.contains(Definitionen.V40)))
				return true;
		}
		return false;
	}

	/**
	 * Gibt das Systemobjekt typ.wvzInhalt zu dem gegebenen Bildinhalt zurck.
	 *  
	 * @param bildinhalt Bildinhalt aus den konfigurieerenden Eingenschaften des WvzInhaltes.
	 * @return Systemobjekt WvzInhalt oder <code>null</code>.
	 */
	public SystemObject getWvzInhalt(String bildinhalt)
	{
		if (m_pidWvzInhalt != null)
		{
			if (m_pidWvzInhalt.containsKey(bildinhalt))
			{
				return m_dataModel.getObject(m_pidWvzInhalt.get(bildinhalt));
			}
		}
		
		return null;
	}
	
	/**
	 * Gibt den AnzeigeTyp der Anzeige zurck.
	 * 
	 * @return AnzeigeTyp der Anzeige.
	 */
	public SystemObject getAnzeigeTyp()
	{
		return m_anzeigeTyp;
	}
	
	/**
	 * Gibt die Pid der Anzeige zurck.
	 * 
	 * @return Pid der Anzeige.
	 */
	public String getPid()
	{
		if (m_anzeige != null)
		{
			return m_anzeige.getPid();
		}
		return null;
	}
	/**
	 * Gibt den Namen der Anzeige zurck.
	 * 
	 * @return Name der Anzeige.
	 */
	public String getName()
	{
		if (m_anzeige != null)
		{
			return m_anzeige.getName();
		}
		return null;
	}

	/**
	 * Gibt die Liste der dieser Anzeige zugeordneten Fahrstreifen zurck.
	 * @return Liste der dieser Anzeige zugeordneten Fahrstreifen.
	 */
	public List<SystemObject> getFahrstreifen()
	{
		return m_fahrstreifen;
	}

	/**
	 * Setzt die Liste der dieser Anzeige zugeordneten Fahrstreifen.
	 * @param fahrstreifen zu setzende Liste der dieser Anzeige zugeordneten Fahrstreifen.
	 */
	public void setFahrstreifen(List<SystemObject> fahrstreifen)
	{
		m_fahrstreifen = fahrstreifen;
	}

	/**
	 * Gibt den Wert der Variable m_lage zurck.</br>
	 * Die Lage 'STNDS' (Standspur) wird auf 'HFS' angepasst.
	 *
	 * @return Wert der Variable m_lage.
	 */
	public String getLage()
	{
		if (m_lage.equals("STNDS"))
			return "HFS";
		
		return m_lage;
	}

	/**
	 * Setzt den Wert der Variable m_lage.
	 *
	 * @param lage zu setzender Wert der Variable m_lage.
	 */
	public void setLage(String lage)
	{
		m_lage = lage;
	}

	/**
	 * Methode fgt den bergebenen Listener zur aktuellen Liste hinzu.
	 * 
	 * @param listener IUpdateAnzeige.
	 */
	public void addListener(IUpdateNeuedaten listener)
	{		
		if (!m_listener.contains(listener))
		{
			m_listener.add(listener);
		}
	}

	/**
	 * Methode lscht einen angemeldeten Listener.
	 * 
	 * @param listener IUpdateAnzeige.
	 */
	public void removeListener(IUpdateNeuedaten listener)
	{
		if (m_listener.contains(listener))
		{
			m_listener.remove(listener);
		}
	}
	
	/**
	 * Methode informiert alle angemeldeten Listener ber die nderung des Parameters.
	 */
	protected void fireEvent()
	{
		for (Iterator<IUpdateNeuedaten> iterator = m_listener.iterator(); iterator.hasNext();)
		{
			IUpdateNeuedaten iUpdNd = (IUpdateNeuedaten)iterator.next();
			
//			System.out.println("IUpdateNeueDaten " + " OBJ " + m_anzeige.getPid() + " " + iUpdNd);
			
			iUpdNd.neueDatenErhalten();
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "Anzeige [m_anzeige=" + m_anzeige + ", m_anzeigeTyp=" + m_anzeigeTyp + ", m_wvzInhalte=" + m_wvzInhalte + ", m_lage=" + m_lage + "]";
	}

}
