/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die Daten der Attributgruppe 'atg.anzeigeQuerschnittEigenschaftIst' des
 * Teilmodell AnzeigenGlobal (kb.tmAnzeigenGlobal)
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class AnzeigeQuerschnittEigenschaftIst extends DaVKommunikation
{
  /**
   * Enum zur Kodierung des Status des Anzeigequerschnitts
   * @author Swarco Traffic Systems GmbH
   * @author Matthias Obert
   */
  public enum Status
  {
    /**
     * Undefiniert
     */
    UNDEFINIERT,  
    
    /**
     * OK
     */
    OK,  
    
    /**
     * Gestrt
     */
    GESTOERT,  
    
    /**
     * Teilstrung
     */
    TEILSTOERUNG,  
    
    /**
     * Stromausfall
     */
    STROMAUSFALL,  
    
    /**
     * Kommunikationsausfall
     */
    KOMMUNIKATIONSAUSFALL,  
    
    /**
     * Trkontakt
     */
    TUERKONTAKT,
  };

  /**
   * Enum zur Kodierung des Betriebszustands des Anzeigequerschnitts
   * @author Swarco Traffic Systems GmbH
   * @author Matthias Obert
   */
  public enum BetriebsZustand
  {
    /**
     * Betriebsart kann nicht ermittelt werden
     */
    UNDEFINIERT,  
    
    /**
     * Normalbetrieb im Sinne der der Definition der TLS
     */
    AQNORMALBETRIEB,  
    
    /**
     * Blindbetrieb im Sinne der Definition der TLS
     */
    AQBLINDBETRIEB,  
    
    /**
     * Vorortbetrieb im Sinne der Definition der TLS
     */
    AQVORORTBETRIEB,  
    
    /**
     * Autarkbetrieb im Sinne der Definition der TLS
     */
    AQAUTARKBETRIEB,  
    
    /**
     * Testbetrieb im Sinne der Definition der TLS
     */
    AQTESTBETRIEB,  
    
    /**
     * Notbetrieb im Sinne der Definition der TLS
     */
    AQNOTBETRIEB,
    
    /**
     * Externe Steuerung
     */
    AQEXTERNESTEUERUNG,
    
    /**
     * Tunnelbetrieb
     */
    AQTUNNELBETRIEB, 
  };

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.anzeigeQuerschnittEigenschaftIst";
  
  /**
   * Anzeigequerschnitt dessen Eigenschaften gelesen werden sollen
   */
  private SystemObject m_anzeigeQuerschnitt;

  /**
   * Anzeigen innerhalb des Anzeigequerschnitts
   */
  private Vector<SystemObject> m_anzeigen = new Vector<SystemObject>();
  
  /**
   * Status des Anzeigequerschnitts
   */
  private Status m_status = Status.UNDEFINIERT;

  /**
   * Referenz auf Fehlermeldungen
   */
  private Vector<SystemObject> m_aktFehlerMeldungen = new Vector<SystemObject>();
  
  /**
   * Helligkeit des Anzeigequerschnitts.
   */
  private int m_helligkeit;
  
  /**
   * Betriebszustand des Anzeigequerschnitts hnlich TLS.
   */
  private BetriebsZustand m_betriebsZustand = BetriebsZustand.UNDEFINIERT;
  
  /**
   * Merker ob der Anzeigequerschnitt passiviert ist
   */
  boolean m_passiviert = false;
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Datenzugriff
   */
  private Object m_mutex = new Object();
  
  /**
   * Kennung fr den Datenempfang 'keine Daten'.
   */
  private boolean m_keineDaten = true;
  /**
   * Kennung fr den Datenempfang 'keine Quelle'.
   */
  private boolean m_keineQuelle = true;
  
  /**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   * @param anzeigeQuerschnitt Anzeige deren Eigenschaften gelesen werden sollen
   */
  public AnzeigeQuerschnittEigenschaftIst( ClientDavConnection connection, SystemObject anzeigeQuerschnitt )
  {
    super( connection );
    
    m_anzeigeQuerschnitt = anzeigeQuerschnitt;

    if (connection != null)
      anmeldeDav(); 
  }

  /**
   * Methode meldet sich fr alle fr den Steuerungskern relevaten Attributgruppen am DaV an
   */
  public void anmeldeDav ()
  {
    // Beim 1. Mal die Daten direkt vom DaV lesen
    
    leseDatenDirektVonDaV ( m_anzeigeQuerschnitt.getPid(), _pidAtg, "asp.aqZustand" );
    
    // Thread starten zum Auswerten der Attribugruppen vom DaV bei nderungen
    
    Thread thread = new Thread(this);
    thread.setName (this.getClass().getName());
    
    thread.start();
    
    // Anmelden am DaV
    
    anmeldeLeseDaV( m_anzeigeQuerschnitt.getPid(), _pidAtg, "asp.aqZustand", ReceiverRole.receiver() );
  }

  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  @Override
  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.anzeigeEigenschaftIst'
   * @param dat vom DaV empfangene Daten
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
    	if (dat.isNoSourceAvailable())
    		m_keineQuelle = true;
    	else
    		m_keineQuelle = false;

    	if (!dat.hasData())
    	{
    		m_keineDaten = true;
    		
  			m_betriebsZustand = BetriebsZustand.UNDEFINIERT;
    		m_helligkeit = -1;
    		m_passiviert = false;
    		m_status = Status.UNDEFINIERT;
    		m_aktFehlerMeldungen.clear();
    	}
    	else
    	{
    		m_keineDaten = false;
    		
    		Data data = dat.getData();

    		// Helligkeit
    		
    		m_helligkeit = data.getItem("Eigenschaft").getUnscaledValue("Helligkeit").intValue();
    		
    		// Betriebszustand
    		
  			m_betriebsZustand = BetriebsZustand.UNDEFINIERT;
    		if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQNormalbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQNORMALBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQBlindbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQBLINDBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQVorortbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQVORORTBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQAutarkbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQAUTARKBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQTestbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQTESTBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQNotbetrieb"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQNOTBETRIEB;
    		}
    		else if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQExterneSteuerung"))
    		{
    			m_betriebsZustand = BetriebsZustand.AQEXTERNESTEUERUNG;
    		}
    		
    		// Passiviert
    		
    		m_passiviert = data.getItem("Eigenschaft").getScaledValue("Passiviert").getValueText().equals("Ja");
    		
    		// Status
    		
    		m_status = Status.UNDEFINIERT;
    		if (data.getScaledValue("Status").getValueText().equals("OK"))
    		{
    			m_status = Status.OK;
    		}
    		else if (data.getScaledValue("Status").getValueText().equals("Gestrt"))
    		{
    			m_status = Status.GESTOERT;
    		}
    		else if (data.getScaledValue("Status").getValueText().equals("Teilstrung"))
    		{
    			m_status = Status.TEILSTOERUNG;
    		}
    		else if (data.getScaledValue("Status").getValueText().equals("Stromausfall"))
    		{
    			m_status = Status.STROMAUSFALL;
    		}
    		else if (data.getScaledValue("Status").getValueText().equals("Kommunikationsausfall"))
    		{
    			m_status = Status.KOMMUNIKATIONSAUSFALL;
    		}
    		else if (data.getScaledValue("Status").getValueText().equals("Trkontakt"))
    		{
    			m_status = Status.TUERKONTAKT;
    		}
    	
    		// aktuelle Fehlermeldungen

    		m_aktFehlerMeldungen.clear();
    		for (int i = 0; i < data.getArray("AktFehlerMeldungen").getLength(); i++)
				{
					m_aktFehlerMeldungen.add(data.getArray("AktFehlerMeldungen").getReferenceValue(i).getSystemObject());
				}
    	}
    }
  }

  /**
   * Methode liefert die Variable m_anzeigeQuerschnitt zurck
   * @return Variable m_anzeigeQuerschnitt
   */
  public SystemObject getAnzeigeQuerschnitt()
  {
    return m_anzeigeQuerschnitt;
  }

  /**
   * Methode setzt die Variable m_anzeigeQuerschnitt
   * @param anzeigeQuerschnitt Wert auf den die Variable m_anzeigeQuerschnitt gesetzt werden soll
   */
  public void setAnzeigeQuerschnitt( SystemObject anzeigeQuerschnitt )
  {
    m_anzeigeQuerschnitt = anzeigeQuerschnitt;
  }

  /**
   * Methode liefert die Variable m_anzeigen zurck
   * @return Variable m_anzeigen
   */
  public Vector<SystemObject> getAnzeigen()
  {
    return m_anzeigen;
  }

  /**
   * Methode setzt die Variable m_anzeigen
   * @param anzeigen Wert auf den die Variable m_anzeigen gesetzt werden soll
   */
  public void setAnzeigen( Vector<SystemObject> anzeigen )
  {
    m_anzeigen = anzeigen;
  }

  /**
   * Methode liefert die Variable m_status zurck
   * @return Variable m_status
   */
  public Status getStatus()
  {
    return m_status;
  }

  /**
   * Methode setzt die Variable m_status
   * @param status Wert auf den die Variable m_status gesetzt werden soll
   */
  public void setStatus( Status status )
  {
    m_status = status;
  }

  /**
   * Methode liefert die Variable m_aktFehlerMeldungen zurck
   * @return Variable m_aktFehlerMeldungen
   */
  public Vector<SystemObject> getAktFehlerMeldungen()
  {
    return m_aktFehlerMeldungen;
  }

  /**
   * Methode setzt die Variable m_aktFehlerMeldungen
   * @param aktFehlerMeldungen Wert auf den die Variable m_aktFehlerMeldungen gesetzt werden soll
   */
  public void setAktFehlerMeldungen( Vector<SystemObject> aktFehlerMeldungen )
  {
    m_aktFehlerMeldungen = aktFehlerMeldungen;
  }

  /**
   * Methode liefert die Variable m_helligkeit zurck
   * @return Variable m_helligkeit
   */
  public int getHelligkeit()
  {
    return m_helligkeit;
  }

  /**
   * Methode setzt die Variable m_helligkeit
   * @param helligkeit Wert auf den die Variable m_helligkeit gesetzt werden soll
   */
  public void setHelligkeit( int helligkeit )
  {
    m_helligkeit = helligkeit;
  }

  /**
   * Methode liefert die Variable m_betriebsZustand zurck
   * @return Variable m_betriebsZustand
   */
  public BetriebsZustand getBetriebsZustand()
  {
    return m_betriebsZustand;
  }

  /**
   * Methode setzt die Variable m_betriebsZustand
   * @param betriebsZustand Wert auf den die Variable m_betriebsZustand gesetzt werden soll
   */
  public void setBetriebsZustand( BetriebsZustand betriebsZustand )
  {
    m_betriebsZustand = betriebsZustand;
  }

  /**
   * Methode liefert die Variable m_passiviert zurck
   * @return Variable m_passiviert
   */
  public boolean isPassiviert()
  {
    return m_passiviert;
  }

  /**
   * Methode setzt die Variable m_passiviert
   * @param passiviert Wert auf den die Variable m_passiviert gesetzt werden soll
   */
  public void setPassiviert( boolean passiviert )
  {
    m_passiviert = passiviert;
  }

	/**
	 * Gibt die Kennung fr den Datenempfang 'keine Daten' zurck.
	 * @return Kennung fr den Datenempfang 'keine Daten'.
	 */
	public boolean isKeineDaten()
	{
		return m_keineDaten;
	}

	/**
	 * Gibt die Kennung fr den Datenempfang 'keine Quelle' zurck.
	 * @return Kennung fr den Datenempfang 'keine Quelle'.
	 */
	public boolean isKeineQuelle()
	{
		return m_keineQuelle;
	}

}
