/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

/**
 * Diese Klasse implementiert die Funktionalitt der Zeitsteuerung bei der Abarbeitung der spontanen
 * Kfz Einzeldaten.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class Zeitsteuerung
{
	/**
	 * Intervalllnge in Sekunden.
	 */
	private long m_intervall;	
	/**
	 * Kennung Zeitsteuerung aktiv.
	 */
	private boolean m_aktiv = false;
	/**
	 * Zugriff auf die Instanz der Zeitsteuerung.
	 */
	private ZeitsteuerungRun m_zeitsteuerungRun;	
	/**
	 * Zugriff auf den Thread der Zeitsteuerung.
	 */
	private Thread m_zeitsteuerungThread;
	/**
	 * Listener fr die Zeitsteuerung.
	 */
	private Vector<IZeitsteuerung> m_listener = new Vector<IZeitsteuerung>();
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public Zeitsteuerung()
	{
	}

	/**
	 * Diese Methode startet einen Timer fr die Abarbeitung der Kfz Einzeldaten.<br>
	 * Luft bereits ein Timer, wird dieser zzuerst gestoppt und anschlieend ein neuer Timer gestartet.
	 * 
	 * @param intervall Intervalllnge in Sekunden
	 */
	public void timerStarten(long intervall)
	{
		if (m_zeitsteuerungRun == null)
		{
			m_zeitsteuerungRun = new ZeitsteuerungRun(intervall);
		}
		else
		{
			m_zeitsteuerungRun.timerBeenden();
			m_zeitsteuerungRun = new ZeitsteuerungRun(intervall);			
		}
		
		if (m_zeitsteuerungThread == null)
		{
			m_zeitsteuerungThread = new Thread(m_zeitsteuerungRun, "ZeitsteuerungRun");			
		}
		
		m_zeitsteuerungThread.interrupt();
		m_zeitsteuerungThread = new Thread(m_zeitsteuerungRun, "ZeitsteuerungRun");			
		m_zeitsteuerungThread.setName("ZeitsteuerungRun " + new Date());
		m_zeitsteuerungThread.start();
		m_aktiv = true;
	}
	
	/**
	 * Aktiven Timer beenden.
	 */
	public void timerBeenden()
	{
		if (m_zeitsteuerungRun != null)
		{
			m_zeitsteuerungRun.timerBeenden();
			m_aktiv = false;
		}
	}
	
	/**
	 * Prft, ob die eingestellte Zeit des Timers abgelaufen ist.
	 * 
	 * @return true wenn die eingestellte Zeit abgelaufen ist, ansonsten false.
	 */
	public boolean isAbgelaufen()
	{
		if (m_zeitsteuerungRun != null)
		{
			if (m_zeitsteuerungRun.isAbgelaufen())
			{
				m_aktiv = false;
				return true;
			}
			else
			{
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * Gibt die Intervalllnge in Sekunden zurck.
	 * @return Intervalllnge in Sekunden.
	 */
	public long getIntervall()
	{
		return m_intervall;
	}

	/**
	 * Setzt die Intervalllnge in Sekunden.
	 * @param intervall zu setzende Intervalllnge in Sekunden.
	 */
	public void setIntervall(long intervall)
	{
		m_intervall = intervall;
	}

	/**
	 * Methode fgt den bergebenen Listener zur aktuellen Liste hinzu.
	 * 
	 * @param listener LangsameFahrzeuge.
	 */
	public void addListener(IZeitsteuerung listener)
	{		
		if (!m_listener.contains(listener))
		{
			m_listener.add(listener);
		}
	}

	/**
	 * Methode lscht einen angemeldeten Listener.
	 * 
	 * @param listener LangsameFahrzeuge.
	 */
	public void removeListener(IZeitsteuerung listener)
	{
		if (m_listener.contains(listener))
		{
			m_listener.remove(listener);
		}
	}
	
	/**
	 * Methode veranlasst eine Neuberechnung der KfzEinzeldaten
	 * fr alle angemeldeten Listener.
	 */
	protected void fireEvent()
	{
		for (Iterator<IZeitsteuerung> iterator = m_listener.iterator(); iterator.hasNext();)
		{
			IZeitsteuerung izs = (IZeitsteuerung)iterator.next();
			
			System.out.println("iZeitsteuerung " + izs);
			
			izs.update();			
		}
	}

	
	/**
	 * Klasse implementiert die Zeitsteuerung fr die Abarbeitung der spontanen Kfz Einzeldaten.
	 *
	 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	private class ZeitsteuerungRun implements Runnable
	{
		/**
		 * Intervalllnge in Millisekunden.
		 */
		private long m_intervall;
		/**
		 * Flag, zum Beenden des Threads.
		 */
		private boolean m_beenden = false;
		/**
		 * Kennung, ob die Zeit abgelaufen ist.
		 */
		private boolean m_abgelaufen = false;
		
		/**
		 * Konstruktor der Klasse.
		 * 
		 * @param intervall Intervalllnge in Sekunden.
		 */
		public ZeitsteuerungRun(long intervall)
		{
			m_intervall = intervall*1000;
		}
		
		@Override
		public void run()
		{
			System.out.println("Timer (Run) neu gestartet Intervall=" + m_intervall);

			long _intervall = m_intervall;
			m_abgelaufen = false;
			
			while (true)
			{
				try
				{						
					Thread.sleep(100);
					_intervall -= 100;
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				
				if ((m_beenden) || (_intervall <= 0))
					break;
			}
			
			if (_intervall <= 0)
			{
				m_abgelaufen = true;
				
				System.out.println("Timer (Run) abgelaufen (" + new Date().toString() + ")");
				
				fireEvent();
			}
			else
			{
				System.out.println("Timer (Run) wegen neuer Daten beendet");
			}
		}

//		/**
//		 * Gibt die Intervalllnge in Millisekunden zurck.
//		 * @return Intervalllnge in Millisekunden.
//		 */
//		public long getIntervall()
//		{
//			return m_intervall;
//		}
//
//		/**
//		 * Setzt die Intervalllnge in Millisekunden.
//		 * @param intervall zu setzende Intervalllnge in Millisekunden.
//		 */
//		public void setIntervall(long intervall)
//		{
//			m_intervall = intervall;
//		}
		
		/**
		 * Methode zum Beenden des Timers.
		 */
		public void timerBeenden()
		{
			m_beenden = true;
		}
		
		/**
		 * Prft, ob die eingestellte Zeit des Timers abgelaufen ist.
		 * 
		 * @return true wenn die eingestellte Zeit abgelaufen ist, ansonsten false.
		 */
		public boolean isAbgelaufen()
		{
			return m_abgelaufen;
		}
	}


	/**
	 * Prft, ob die Zeitsteuerung aktiv ist.
	 * @return true wenn Zeitsteuerung aktiv, ansonsten false.
	 */
	public boolean isAktiv()
	{
		return m_aktiv;
	}
}
